@@ -36,7 +36,7 @@ public:
 	void setText(QString title, QString text);
 #ifdef Q_WS_MAEMO_5		// workaround for buggy Qt autorotation
 	int exec() {
-		QRect r = static_cast<QDialog*>(this->parent())->rect();
+		QRect r = dynamic_cast<QWidget*>(this->parent())->rect();
 		if(r.width() < r.height())
 			this->setAttribute(Qt::WA_Maemo5PortraitOrientation);
 		int e = QDialog::exec();
