@@ -39,7 +39,7 @@ PackageSelector::PackageSelector(Package* pkg, AAptInterface* apt, QWidget *pare
 	if( getMaemoOrgUrl(pkg)=="" && getMaemoOrgUrl(pkg->availablePackage())=="" ) {
 		ui->pushButton_website->setEnabled(false);
 	}
-	if( pkg->isInstalled() || pkg->isMarkedForOperation() )
+	if( pkg->isMarkedForOperation() )
 		ui->pushButton_blacklist->setEnabled(false);
 
 	if( pkg->icon() && !pkg->icon()->isNull() ) {
@@ -54,105 +54,20 @@ PackageSelector::PackageSelector(Package* pkg, AAptInterface* apt, QWidget *pare
 	else
 		ui->label_header->setText("<b>" + pkg->name() + "</b>");
 
-	updateHeader2();
+	ui->radioTabBasic->setChecked(true);
+	updateInfo();
 
-	QString longtext;
-
-	longtext += "Category: " + pkg->section() + "<br>";
-
-	if( pkg->isInstalled() ) {
-		longtext += "Installation date: ";
-		if( pkg->date().isValid() )
-			longtext += pkg->date().toString("yyyy-MM-dd hh:mm");
-		else
-			longtext += "Unknown";
-		longtext += "<br>";
-	}
-
-	if( pkg->isInstalled() && pkg->installedSize() > 0 )
-		longtext += QString("Size: %L1 kB<br>").arg(pkg->installedSize() );
-
-
-	if( !pkg->isInstalled() ){
-		longtext += "Available version <b>" + pkg->version() + "</b>";
-		if( pkg->size() > 0 )
-			longtext += QString(" (%L1 kB)").arg(pkg->size()/1024 );
-		longtext += "<br>";
-	}
-	else if( pkg->isUpgradeable() ){
-		longtext += "Available version <b>" + pkg->upgradeableVersion() + "</b>";
-		if( pkg->size() > 0 )
-			longtext += QString(" (%L1 kB)").arg(pkg->size()/1024 );
-		longtext += "<br>";
-	}
-
-	if( !pkg->isInstalled() ) {
-		longtext += "Available package date: ";
-		if( pkg->date().isValid() )
-			longtext += pkg->date().toString("yyyy-MM-dd hh:mm");
-		else
-			longtext += "Unknown";
-		longtext += "<br>";
-	} else if( pkg->isUpgradeable() ) {
-		longtext += "Available package date: ";
-		if( pkg->availablePackage() && pkg->availablePackage()->date().isValid() )
-			longtext += pkg->availablePackage()->date().toString("yyyy-MM-dd hh:mm");
-		else
-			longtext += "Unknown";
-		longtext += "<br>";
-	}
-
-	if( !pkg->isInstalled() )
+	// shorter button labels for portait mode
+	if( QApplication::desktop()->width() < QApplication::desktop()->height() )
 	{
-		longtext += "Repositories: ";
-		if( pkg->repositories().count()>0 ) {
-			for(int i=0; i<pkg->repositories().count(); i++ ) {
-				if( pkg->repositories().at(i) )
-					longtext += pkg->repositories().at(i)->name();
-				else
-					longtext += "unknown";
-				if( i<pkg->repositories().count()-1 )
-					longtext += ", ";
-			}
-			longtext += "<br>";
-		} else {
-			longtext += "unknown<br>";
-		}
+		ui->radioTabBasic->setText("Info");
+		ui->radioTabChanges->setText("Changes");
+		ui->radioTabDepends->setText("Deps");
 	}
 
-	if( pkg->isUpgradeable() )
-	{
-		longtext += "Repositories: ";
-		Package* upg_pkg = pkg->availablePackage();
-		if( upg_pkg && upg_pkg->repositories().count()>0 ) {
-			for(int i=0; i<upg_pkg->repositories().count(); i++ ) {
-				if( upg_pkg->repositories().at(i) )
-					longtext += upg_pkg->repositories().at(i)->name();
-				else
-					longtext += "unknown";
-				if( i<upg_pkg->repositories().count()-1 )
-					longtext += ", ";
-			}
-			longtext += "<br>";
-		} else {
-			longtext += "unknown<br>";
-		}
-	}
-
-	longtext += "<br>";
-	longtext += pkg->descShort();
-	QString descLong = pkg->descLong();
-	if( descLong.length()>0 ) {
-		descLong.replace('\n',"<br>");
-		longtext += "<br><font size=\"-1\"><br>" + descLong + "</font><br>";
-	}
-
-
-	pkg->depends();
-	pkg->conflicts();
-
-
-	ui->label_text->setText( longtext );
+	Package* upg_pkg = iPkg->availablePackage();
+	if( !pkg->isUpgradeable() || (upg_pkg && upg_pkg->upgradeDescription()=="") )
+		ui->radioTabChanges->hide();
 
 	if( pkg->isUpgradeable() )
 		ui->radioInstall->setText("Upgrade");
@@ -179,10 +94,6 @@ PackageSelector::PackageSelector(Package* pkg, AAptInterface* apt, QWidget *pare
 		ui->label_statusicon->setPixmap(QPixmap(":/icons/icons/pkg_remove.png"));
 		ui->radioRemove->setChecked(true);
 	}
-
-#ifndef Q_WS_MAEMO_5	// for simulator debug
-	ui->radioInstall->setChecked(true);
-#endif
 }
 
 PackageSelector::~PackageSelector()
@@ -217,7 +128,7 @@ void PackageSelector::on_pushButton_blacklist_clicked()
 	BlacklistSelect s(iPkg, this);
 	BlacklistSelect::blackList old = iPkg->blacklisted();
 	if( s.exec() ) {
-		updateHeader2();
+		updateInfo();
 		if( old != iPkg->blacklisted() ) {
 			iAptInterface->removeFromBlacklist(iPkg, old);
 			iAptInterface->writeBlacklist();
@@ -243,8 +154,38 @@ QString PackageSelector::getMaemoOrgUrl(Package* pkg)
 	return url;
 }
 
-void PackageSelector::updateHeader2()
+void PackageSelector::updateInfo()
 {
+	if( ui->radioTabBasic->isChecked() )
+		on_radioTabBasic_clicked();
+	else if( ui->radioTabChanges->isChecked() )
+		on_radioTabChanges_clicked();
+	else if( ui->radioTabDepends->isChecked() )
+		on_radioTabDepends_clicked();
+}
+
+void PackageSelector::on_pushButton_website_clicked()
+{
+	QString url = getMaemoOrgUrl(iPkg);
+	if( url == "" ) {
+		url = getMaemoOrgUrl(iPkg->availablePackage());
+		if( url == "")
+			return;
+	}
+
+	QDBusConnection conn = QDBusConnection::connectToBus(QDBusConnection::SessionBus, "faster_application_manager");
+
+	QString service = "com.nokia.osso_browser";
+	QString path = "/com/nokia/osso_browser/request";
+	QString method = "open_new_window";
+
+	QDBusInterface browser(service, path, service, conn, this);
+	browser.call(method, url);
+}
+
+void PackageSelector::on_radioTabBasic_clicked()
+{
+	ui->label_header2->show();
 	QString header2;
 
 	if( iPkg->isInstalled() ) {
@@ -257,21 +198,22 @@ void PackageSelector::updateHeader2()
 	if( iPkg->markedOperation() == Package::PkgOpInstallUpgrade )
 	{
 		if( iPkg->isUpgradeable() )
-			header2 += "Marked for <b>upgrade</b>";
+			header2 += "Marked for <b>upgrade</b><br>";
 		else
-			header2 += "Marked for <b>installation</b>";
+			header2 += "Marked for <b>installation</b><br>";
 	} else if(iPkg->markedOperation() == Package::PkgOpRemove)
 	{
-		header2 += "Marked for <b>removal</b>";
+		header2 += "Marked for <b>removal</b><br>";
 	}
 
-	if(iPkg->isBlacklisted()) {
-		header2 = "BLACKLISTED";
+	if( iPkg->isBlacklisted() ) {
+		header2 += "BLACKLISTED";
 		if( iPkg->blacklisted() == BlacklistSelect::BlacklistAll )
 			header2 += " (all)";
 		else if( iPkg->blacklisted() == BlacklistSelect::BlacklistThis )
 			header2 += " (this)";
 		iPkg->setMarkedForOperation(Package::PkgOpNone);
+		header2 += "<br>";
 	}
 
 	ui->radioInstall->setEnabled(true);
@@ -294,23 +236,137 @@ void PackageSelector::updateHeader2()
 	}
 
 	ui->label_header2->setText(header2);
-}
 
-void PackageSelector::on_pushButton_website_clicked()
-{
-	QString url = getMaemoOrgUrl(iPkg);
-	if( url == "" ) {
-		url = getMaemoOrgUrl(iPkg->availablePackage());
-		if( url == "")
-			return;
+
+	QString longtext;
+	longtext += "Category: " + iPkg->section() + "<br>";
+
+	if( iPkg->isInstalled() ) {
+		longtext += "Installation date: ";
+		if( iPkg->date().isValid() )
+			longtext += iPkg->date().toString("yyyy-MM-dd hh:mm");
+		else
+			longtext += "Unknown";
+		longtext += "<br>";
 	}
 
-	QDBusConnection conn = QDBusConnection::connectToBus(QDBusConnection::SessionBus, "faster_application_manager");
+	if( iPkg->isInstalled() && iPkg->installedSize() > 0 )
+		longtext += QString("Size: %L1 kB<br>").arg(iPkg->installedSize() );
 
-	QString service = "com.nokia.osso_browser";
-	QString path = "/com/nokia/osso_browser/request";
-	QString method = "open_new_window";
 
-	QDBusInterface browser(service, path, service, conn, this);
-	browser.call(method, url);
+	if( !iPkg->isInstalled() ){
+		longtext += "Available version <b>" + iPkg->version() + "</b>";
+		if( iPkg->size() > 0 )
+			longtext += QString(" (%L1 kB)").arg(iPkg->size()/1024 );
+		longtext += "<br>";
+	}
+	else if( iPkg->isUpgradeable() ){
+		longtext += "Available version <b>" + iPkg->upgradeableVersion() + "</b>";
+		if( iPkg->size() > 0 )
+			longtext += QString(" (%L1 kB)").arg(iPkg->size()/1024 );
+		longtext += "<br>";
+	}
+
+	if( !iPkg->isInstalled() ) {
+		longtext += "Available package date: ";
+		if( iPkg->date().isValid() )
+			longtext += iPkg->date().toString("yyyy-MM-dd hh:mm");
+		else
+			longtext += "Unknown";
+		longtext += "<br>";
+	} else if( iPkg->isUpgradeable() ) {
+		longtext += "Available package date: ";
+		if( iPkg->availablePackage() && iPkg->availablePackage()->date().isValid() )
+			longtext += iPkg->availablePackage()->date().toString("yyyy-MM-dd hh:mm");
+		else
+			longtext += "Unknown";
+		longtext += "<br>";
+	}
+
+	if( !iPkg->isInstalled() )
+	{
+		longtext += "Repositories: ";
+		if( iPkg->repositories().count()>0 ) {
+			for(int i=0; i<iPkg->repositories().count(); i++ ) {
+				if( iPkg->repositories().at(i) )
+					longtext += iPkg->repositories().at(i)->name();
+				else
+					longtext += "unknown";
+				if( i<iPkg->repositories().count()-1 )
+					longtext += ", ";
+			}
+			longtext += "<br>";
+		} else {
+			longtext += "unknown<br>";
+		}
+	}
+
+	if( iPkg->isUpgradeable() )
+	{
+		longtext += "Repositories: ";
+		Package* upg_pkg = iPkg->availablePackage();
+		if( upg_pkg && upg_pkg->repositories().count()>0 ) {
+			for(int i=0; i<upg_pkg->repositories().count(); i++ ) {
+				if( upg_pkg->repositories().at(i) )
+					longtext += upg_pkg->repositories().at(i)->name();
+				else
+					longtext += "unknown";
+				if( i<upg_pkg->repositories().count()-1 )
+					longtext += ", ";
+			}
+			longtext += "<br>";
+		} else {
+			longtext += "unknown<br>";
+		}
+	}
+
+	longtext += "<br>" + iPkg->descShort();
+	QString descLong = iPkg->descLong();
+	if( descLong.length()>0 ) {
+		descLong.replace('\n',"<br>");
+		longtext += "<font size=\"-1\"><br><br>" + descLong + "</font>";
+	}
+
+	ui->label_text->setText( longtext );
+}
+
+void PackageSelector::on_radioTabChanges_clicked()
+{
+	ui->label_header2->show();
+	ui->label_header2->setText("Upgrade description:");
+	ui->label_text->setText("");
+
+	Package* upg_pkg = iPkg->availablePackage();
+	if( !upg_pkg )
+		upg_pkg = iPkg;
+
+	if( upg_pkg ) {
+		QString udesc = "<br><font size=\"-1\">";
+		udesc += upg_pkg->upgradeDescription();
+		udesc += "</font>";
+		udesc.replace('\n',"<br>");
+		ui->label_text->setText(udesc);
+	}
+}
+
+void PackageSelector::on_radioTabDepends_clicked()
+{
+	ui->label_header2->hide();
+	ui->label_header2->setText("");
+	ui->label_text->setText("");
+
+	QString text;
+	text += "<b>Depends:</b><br><font size=\"-1\">";
+	text += iPkg->depends().join(", ");
+	if( iPkg->depends().count()==0 )
+		text += "-";
+	text += "<br></font><br>";
+
+	text += "<b>Conflicts:</b><br><font size=\"-1\">";
+	text += iPkg->conflicts().join(", ");
+	if( iPkg->conflicts().count()==0)
+		text += "-";
+	text += "</font>";
+
+	ui->label_text->setText(text);
 }
