@@ -29,6 +29,7 @@
 #include "logview.h"
 #include "sortselector.h"
 #include "settings.h"
+#include "searchoptions.h"
 
 PackageListWidgetItem::PackageListWidgetItem(Package* p_, QString name_) : QListWidgetItem(name_)
 {
@@ -240,6 +241,11 @@ PackageView::PackageView(QWidget *parent) : QMainWindow(parent), ui(new Ui::Pack
 
 	iSortNoticeShown = false;
 
+	iSearchPkgName = false;
+	iSearchDisplayName = true;
+	iSearchDescShort = false;
+	iSearchDescLong = false;
+
 	// fine-tune kinetic scrolling parameters
 	QAbstractKineticScroller* listscroller = ui->listWidget->property("kineticScroller").value<QAbstractKineticScroller*>();
 	if( listscroller )
@@ -277,11 +283,15 @@ bool PackageView::doFilterCategory(Package* pkg)
 	if( pkg->section()=="user/hidden" )
 		return false;
 
-	if( pkg->isBlacklisted() && iSelectedCatFilter != CatFilterBlacklisted )
-		return false;
-	else if ( pkg->isBlacklisted() && iSelectedCatFilter == CatFilterBlacklisted )
-		return true;
-	if( iSelectedCatFilter == CatFilterBlacklisted )
+	if( pkg->isBlacklisted() )
+	{
+		if( iSelectedCatFilter == CatFilterBlacklisted )
+			return true;
+		if( pkg->isInstalled() && (iSelectedStatFilter==Package::PkgStatInstalled || iSelectedStatFilter==Package::PkgStatUnknown) )
+			return true;
+		if( iSelectedCatFilter != CatFilterBlacklisted )
+			return false;
+	} else if( !pkg->isBlacklisted() && iSelectedCatFilter == CatFilterBlacklisted )
 		return false;
 
 	if( iSelectedCatFilter==CatFilterAllMarked ) {
@@ -298,10 +308,12 @@ bool PackageView::doFilterCategory(Package* pkg)
 
 QString PackageView::generateSortString(Package* pkg)
 {
-	// default sort order is alphabetical by name
-	QString sortstr = pkg->displayName();
+	QString sortstr;
 
-	if( iSortOrder==SortDateDesc ) {
+	if( iSortOrder==SortAlpha ) {
+		sortstr = pkg->displayName();
+	}
+	else if( iSortOrder==SortDateDesc ) {
 		Package* upg = 0;
 		if( pkg->isUpgradeable() )
 			upg = pkg->availablePackage();
@@ -315,7 +327,7 @@ QString PackageView::generateSortString(Package* pkg)
 			iPackagesEmptySort++;
 		}
 	}
-	if( iSortOrder==SortSizeDesc ) {
+	else if( iSortOrder==SortSizeDesc ) {
 		if( pkg->isInstalled() )
 			sortstr = QString("%1").arg(pkg->installedSize()*1024, 12 );
 		else
@@ -596,12 +608,12 @@ void PackageView::on_listWidget_itemClicked(QListWidgetItem* item)
 	PackageSelector s(pkg, iAptInterface, this);
 	s.exec();
 
+	pkg->setMarkedForOperation( s.selectedOperation() );
+	item->setData(UserRoleMarked, (int)s.selectedOperation());
+	updateLabel();
+
 	if( pkg->isBlacklisted() != bl ) {
 		openWin();
-	} else {
-		pkg->setMarkedForOperation( s.selectedOperation() );
-		item->setData(UserRoleMarked, (int)s.selectedOperation());
-		updateLabel();
 	}
 }
 
@@ -666,6 +678,8 @@ void PackageView::on_btn_StatusFilter_clicked()
 	FilterSelect f("Status filter", this);
 	f.setList( iStatFilterLabels, iSelectedStatFilter );
 
+	int oldfilter = iSelectedStatFilter;
+
 	bool s = f.exec();
 
 	if( s ) {
@@ -678,6 +692,20 @@ void PackageView::on_btn_StatusFilter_clicked()
 		iSortNoticeShown = false;
 
 		openWin();
+
+		if( oldfilter==Package::PkgStatInstalled && iSelectedStatFilter!=Package::PkgStatInstalled &&
+			iAptInterface->needListOrDateRefresh() )
+		{
+			iMainWindow->setNextOperation(MainWindow::OpOpenPkgView);
+			iMainWindow->busyDialog(true,"Operation in progress","Reading the rest of the package lists");
+
+			if( iAptInterface->needRepoRefresh() )
+				iAptInterface->addQueuedOperation(AAptInterface::ModeAptGetUpdate);
+
+			iAptInterface->addQueuedOperation(AAptInterface::ModeReadPackages);
+			iAptInterface->addQueuedOperation(AAptInterface::ModeFetchDates);
+			iAptInterface->run(iDimmer);
+		}
 	}
 }
 
@@ -772,16 +800,26 @@ void PackageView::on_lineEdit_textEdited(QString text)
 
 	if( text.startsWith(":") ) {
 		for( int i=0; i<packages.count(); i++ ) {
-			if( packages.at(i) && ( packages.at(i)->displayName().startsWith(text.mid(1), Qt::CaseInsensitive) ))
-			{
-				iSearchResults.append( packages.at(i) );
+			if( packages.at(i) ) {
+				if( (iSearchPkgName && packages.at(i)->name().startsWith(text.mid(1), Qt::CaseInsensitive)) ||
+					(iSearchDisplayName && packages.at(i)->displayName().startsWith(text.mid(1), Qt::CaseInsensitive)) ||
+					(iSearchDescShort && packages.at(i)->descShort().startsWith(text.mid(1), Qt::CaseInsensitive)) ||
+					(iSearchDescLong && packages.at(i)->descLong().startsWith(text.mid(1), Qt::CaseInsensitive)) )
+				{
+					iSearchResults.append( packages.at(i) );
+				}
 			}
 		}
 	} else {
 		for( int i=0; i<packages.count(); i++ ) {
-			if( packages.at(i) && ( packages.at(i)->displayName().contains(text, Qt::CaseInsensitive) ))
-			{
-				iSearchResults.append( packages.at(i) );
+			if( packages.at(i) ) {
+				if( (iSearchPkgName && packages.at(i)->name().contains(text, Qt::CaseInsensitive)) ||
+					(iSearchDisplayName && packages.at(i)->displayName().contains(text, Qt::CaseInsensitive)) ||
+					(iSearchDescShort && packages.at(i)->descShort().contains(text, Qt::CaseInsensitive)) ||
+					(iSearchDescLong && packages.at(i)->descLong().contains(text, Qt::CaseInsensitive)) )
+				{
+					iSearchResults.append( packages.at(i) );
+				}
 			}
 		}
 	}
@@ -815,6 +853,29 @@ bool KeyEventGrabber::eventFilter(QObject *obj, QEvent *event)
 	return QObject::eventFilter(obj, event);
 }
 
+void PackageView::on_pushButton_searchOptions_clicked()
+{
+	SearchOptions s(this);
+	s.setSelections(iSearchPkgName, iSearchDisplayName, iSearchDescShort, iSearchDescLong);
+	if( s.exec() )
+	{
+		iSearchPkgName = s.searchPkgName();
+		iSearchDisplayName = s.searchDisplayName();
+		iSearchDescShort = s.searchDescShort();
+		iSearchDescLong = s.searchDescLong();
+
+		on_lineEdit_textEdited( ui->lineEdit->text() );
+	}
+}
+
+void PackageView::setSearchOptions(bool pkgname, bool dispname, bool dshort, bool dlong)
+{
+	iSearchPkgName = pkgname;
+	iSearchDisplayName = dispname;
+	iSearchDescShort = dshort;
+	iSearchDescLong = dlong;
+}
+
 void PackageView::on_actionUpgrade_all_triggered()
 {
 	for( int i=0; i<ui->listWidget->count(); i++ )
