@@ -94,6 +94,7 @@ public:
 	void setSortOrder(sortOrder s_) { iSortOrder = s_; }
 	void clearSelections();
 	void setSearchText(QString text);
+	void setSearchOptions(bool pkgname, bool dispname, bool dshort, bool dlong);
 
 #ifdef Q_WS_MAEMO_5		// workaround for buggy Qt autorotation
 	void show() {
@@ -159,10 +160,16 @@ private:
 	QString iPrevSearchText;
 	KeyEventGrabber* iKeyFilter;
 
+	bool iSearchPkgName;
+	bool iSearchDisplayName;
+	bool iSearchDescShort;
+	bool iSearchDescLong;
+
 	int iPackagesEmptySort;
 
 private slots:
-	void on_actionRestore_all_triggered();
+	void on_pushButton_searchOptions_clicked();
+ void on_actionRestore_all_triggered();
  void on_actionSave_selections_triggered();
 	void on_actionLoad_selections_triggered();
 	void on_btn_Sort_clicked();
